# Persistence

The installed application INETAPP heavily depends on the persistence to store vital information. This information includes, e.g. configuration and user settings. The persistence can be stored in one of the following locations based on the scenario INETAPP is used in.

  * **File System**
  * **MongoDB** [https://www.mongodb.com/what-is-mongodb](https://www.mongodb.com/what-is-mongodb)
  * **Azure CosmosDB SQL-API** [https://docs.microsoft.com/en-us/azure/cosmos-db/](https://docs.microsoft.com/en-us/azure/cosmos-db/)

The default persistence is the *File System*. It suffices use cases of a local server installation with a fixed hard drive. *MongoDB* can be used in cloud based environments where no hard drive is available or the file system can disappear together with the server instance.

The persistence should be selected depending on the usage scenario.

  * The *File System* persistence can be used in every (server-) environment given that the underlying hard disk is always available (e.g. bare bone server, Docker with mounted directories)
  * The *MongoDB* persistence is a hard disk backed persistence that can handle larger chunks of data easily. It should be used in cloud environments in a scenario that requires a large amount of data being stored. This persistence allows to preserve the server configuration in environments where server can be spun up and down at will.
  * The *CosmosDB* persistence is an alternative, cloud based persistence created by Microsoft for Azure deployment.

## File System

The File System persistence is the default persistence of the INETAPP server. There is no further configuration required. The persistence will be stored in the filesystem depending on the operating system:

  * **Windows**: `C:/ProgramData/i-net software`
  * **Linux**: `%USER_HOME%/.i-net software`
  * **macOS**: `%USER_HOME%/.i-net software`

The persistence can be moved if needed using the [Maintenance Module](/documentation/shared/plugins/maintenance/maintenance/en/programdata).

Note: The *File System* persistence can only be used with a single server and must not be shared with another instance.

### Server Configuration / Preferences

The server configuration is not being stored in the File System backed persistence. Depending on which user runs the server the preferences are stored in the following places:

  * **Windows**: The Registry
  * **Linux**: `%USER_HOME%/.java` or `/etc/.java` (System Scope)
  * **macOS**: `~/Library/Preferences/com.inet.*.plist` or `/Library/Preferences/com.inet.*.plist` (for System scope)

## MongoDB

The MongoDB persistence has to be set up with a URI in the following form (for more details [see here](https://docs.mongodb.com/manual/reference/connection-string/)):

```properties
"mongodb://(<USERNAME>:<PASSWORD>@)<MONGODB SERVER>/<DATABASE>"
```

Note: The authentication using `<USERNAME>:<PASSWORD>` is optional.

Note: If the given `<DATABASE>` does not exist, it will be created.

Note: The following versions of MongoDB are currently supported: `2.6` up to `4.0`.

Have a look at [https://github.com/mongodb/mongo-java-driver](https://github.com/mongodb/mongo-java-driver) for additional information 

### Memory Usage Considerations

The following numbers will allow to get an insight on how much memory is required:

| Component                   | Memory used                                  | Comment                                      |
---|---|---| Configuration               | 50KB                                         |                                              
 Eventlog                    | 50MB                                         | Can be reduced in the confirmation settings  
 User Metadata               | 10K per User                                 | mostly depends on the Avatar size            
 User Data (uploaded files)  | Size of compared documents x 3 (for result)  | Depends on the quota                         

## Azure CosmosDB SQL-API

The connection information for the CosmosDB can be obtained, e.g. from the explorer interface.

```properties
"AccountEndpoint=<CosmosDB Server>;AccountKey=<Account Key>;DatabaseName=<Database Name>"
```

  * **CosmosDB Server** - Server Connection URL, including port, e.g. `https://cosmos.company.com:8081/`
  * **Account Key** - the private key required to authenticate against the server
  * **Database Name** - a non standard property to indicate a database name

Note: The connection URL string has to be set in full for the `-Dinet_persistence=` property - see below.

### Memory Usage Considerations

The following numbers will allow to get an insight on how much memory is required:

| Component                   | Memory used                                  | Comment                                      |
---|---|---| Configuration               | 50KB                                         |                                              
 Eventlog                    | unknown                                         | Eventlog is fixed to 30 days of event data  
 User Metadata               | 10K per User                                 | mostly depends on the Avatar size            
 User Data (uploaded files)  | Size of compared documents x 3 (for result)  | Depends on the quota                         

An Azure CosmosDB database will be created with about a dozen container subsets. The commonly used subset is `files`.

INETAPP does not continuously use data - but rather has high peek values when uploading and downloading files and re-indexing caches. The actual data throughput very much depends on the user behaviour as well.

That is why we tend to recommend either using a serverless CosmosDB account or an automatic scaling mode rather than a fixed RU/s mode.

Additional information about Azur CosmosDB pricing can be found here: [https://azure.microsoft.com/en-us/pricing/details/cosmos-db/](https://azure.microsoft.com/en-us/pricing/details/cosmos-db/)

## Installation

Using the *MongoDB* or *CosmosDB* persistence requires additional components to be installed into the `plugins/persistence` subfolder of the installation.

If the respective files do not yet exist, you have to download them from the public maven repository at:

  * **Persistence main jar**: [release version](https://maven.inetsoftware.de/release/de/inetsoftware/persistence)
  * **MongoDB driver package**: [release version](https://maven.inetsoftware.de/release/de/inetsoftware/persistence.mongodb)
  * **CosmosDB driver package**: [release version](https://maven.inetsoftware.de/release/de/inetsoftware/persistence.cosmosdb)

Note: You have to navigate and download the version matching your installation.

Note: Both driver packages, *MongoDB* and *CosmosDB*, have to be extracted into the `plugins/persistence` subfolder. The *persistences main jar* has to be saved into this directory without extracting it.

Note: In addition to the released version, there may be beta versions available: [Persistence beta version](https://maven.inetsoftware.de/beta/de/inetsoftware/persistence), [MongoDB beta version package](https://maven.inetsoftware.de/beta/de/inetsoftware/persistence.mongodb), [CosmosDB beta version package](https://maven.inetsoftware.de/beta/de/inetsoftware/persistence.cosmosdb)

## Configuration of the Persistence

The persistence is the first thing to be initialized when starting the server - before the actual configuration can be set. That is why the persistence has to be set up from the outside using one of the following methods.

### Properties File

A properties file can be created in the `plugins/persistence/` subfolder of the installation. It has to have the name `persistence.properties` with a single entry line:

```properties
inet_persistence=<PERSISTENCE URI>
```

With `<PERSISTENCE URI>` being the URI specific to the persistence. (see above)

### System Property

The system property has to be set when starting the INETAPP server, eg:

```bash
java -jar ... -Dinet_persistence=<PERSISTENCE URI>
```

With `<PERSISTENCE URI>` being the URI specific to the persistence. (see above)

### Environment Variable

The server will check for an environment variable of the name `inet_persistence` of neither properties file or system property are given. On Unix/Linux systems the variable could be set using:

```bash
export inet_persistence=<PERSISTENCE URI>
```

With `<PERSISTENCE URI>` being the URI specific to the persistence. (see above)

### Switching the persistence layer

Switching the persistence layer is not a standard task and should be considered carefully. To switch the persistence, make a full backup of INETAPP server using the Maintenance module and download the backup information.

You can then set up the server with the newly selected persistence store, upload the backup and restore it.

Note: switching to a non-file persistence with custom translations in place requires extra work to distribute the translations. For more information see: [Translations in the cloud](http://docs.inetsoftware.de/reporting/help/translations-in-the-cloud)

